Ddoc

$(SPEC_S Interfacing to C,

	D is designed to fit comfortably with a C compiler for the target
	system. D makes up for not having its own VM by relying on the
	target environment's C runtime library. It would be senseless to
	attempt to port to D or write D wrappers for the vast array of C APIs
	available. How much easier it is to just call them directly.
	<p>

	This is done by matching the C compiler's data types, layouts,
	and function call/return sequences.

<h2>Calling C Functions</h2>

	C functions can be called directly from D. There is no need for
	wrapper functions, argument swizzling, and the C functions do not
	need to be put into a separate DLL.
	<p>

	The C function must be declared and given a calling convention,
	most likely the "C" calling convention, for example:

------
extern (C) int strcmp(char* string1, char* string2);
------

	and then it can be called within D code in the obvious way:

------
import std.string;
int myDfunction(char[] s)
{
    return strcmp(std.string.toStringz(s), "foo");
}
------

	There are several things going on here:

	$(UL 
	<li> D understands how C function names are "mangled" and the
	correct C function call/return sequence.

	<li> C functions cannot be overloaded with another C function
	with the same name.

	<li> There are no __cdecl, __far, __stdcall, __declspec, or other
	such C type modifiers in D. These are handled by attributes, such
	as $(TT extern (C)).

	<li> There are no const or volatile type modifiers in D. To declare
	a C function that uses those type modifiers, just drop those
	keywords from the declaration.

	<li> Strings are not 0 terminated in D. See "Data Type Compatibility"
	for more information about this. However, string literals in D are
	0 terminated.
	)

	C code can correspondingly call D functions, if the D functions
	use an attribute that is compatible with the C compiler, most likely
	the extern (C):

------
// myfunc() can be called from any C function
extern (C)
{
    void myfunc(int a, int b)
    {
	...
    }
}
------

<h2>Storage Allocation</h2>

	C code explicitly manages memory with calls to malloc() and
	free(). D allocates memory using the D garbage collector,
	so no explicit free's are necessary.
	<p>

	D can still explicitly allocate memory using c.stdlib.malloc()
	and c.stdlib.free(), these are useful for connecting to C
	functions that expect malloc'd buffers, etc.
	<p>

	If pointers to D garbage collector allocated memory are passed to
	C functions, it's critical to ensure that that memory will not
	be collected by the garbage collector before the C function is
	done with it. This is accomplished by:

	$(UL 

	<li> Making a copy of the data using c.stdlib.malloc() and passing
	the copy instead.

	<li> Leaving a pointer to it on the stack (as a parameter or
	automatic variable), as the garbage collector will scan the stack.

	<li> Leaving a pointer to it in the static data segment, as the
	garbage collector will scan the static data segment.

	<li> Registering the pointer with the garbage collector with the
	gc.addRoot() or gc.addRange() calls.

	)

	An interior pointer to the allocated memory block is sufficient
	to let the GC
	know the object is in use; i.e. it is not necessary to maintain
	a pointer to the beginning of the allocated memory.
	<p>

	The garbage collector does not scan the stacks of threads not
	created by the D Thread interface. Nor does it scan the data
	segments of other DLL's, etc.

<h2>Data Type Compatibility</h2>

<table border=1 cellspacing=0 cellpadding=5>

	<tr>
	<th>D type</th>
	<th>C type</th>
	</tr>

	<tr>
	<td>$(B void)</td>
	<td>$(B void)</td>
	</tr>

	<tr>
	<td>$(B bit)</td>
	<td>no equivalent</td>
	</tr>

	<tr>
	<td>$(B byte)</td>
	<td>$(B signed char)</td>
	</tr>

	<tr>
	<td>$(B ubyte)</td>
	<td>$(B unsigned char)</td>
	</tr>

	<tr>
	<td>$(B char)</td>
	<td>$(B char) (chars are unsigned in D)</td>
	</tr>

	<tr>
	<td>$(B wchar)</td>
	<td>$(B wchar_t) (when sizeof(wchar_t) is 2)</td>
	</tr>

	<tr>
	<td>$(B dchar)</td>
	<td>$(B wchar_t) (when sizeof(wchar_t) is 4)</td>
	</tr>

	<tr>
	<td>$(B short)</td>
	<td>$(B short)</td>
	</tr>

	<tr>
	<td>$(B ushort)</td>
	<td>$(B unsigned short)</td>
	</tr>

	<tr>
	<td>$(B int)</td>
	<td>$(B int)</td>
	</tr>

	<tr>
	<td>$(B uint)</td>
	<td>$(B unsigned)</td>
	</tr>

	<tr>
	<td>$(B long)</td>
	<td>$(B long long)</td>
	</tr>

	<tr>
	<td>$(B ulong)</td>
	<td>$(B unsigned long long)</td>
	</tr>

	<tr>
	<td>$(B float)</td>
	<td>$(B float)</td>
	</tr>

	<tr>
	<td>$(B double)</td>
	<td>$(B double)</td>
	</tr>

	<tr>
	<td>$(B real)</td>
	<td>$(B long double)</td>
	</tr>

	<tr>
	<td>$(B ifloat)</td>
	<td>$(B float _Imaginary)</td>
	</tr>

	<tr>
	<td>$(B idouble)</td>
	<td>$(B double _Imaginary)</td>
	</tr>

	<tr>
	<td>$(B ireal)</td>
	<td>$(B long double _Imaginary)</td>
	</tr>

	<tr>
	<td>$(B cfloat)</td>
	<td>$(B float _Complex)</td>
	</tr>

	<tr>
	<td>$(B cdouble)</td>
	<td>$(B double _Complex)</td>
	</tr>

	<tr>
	<td>$(B creal)</td>
	<td>$(B long double _Complex)</td>
	</tr>

	<tr>
	<td>$(B struct)</td>
	<td>$(B struct)</td>
	</tr>

	<tr>
	<td>$(B union)</td>
	<td>$(B union)</td>
	</tr>

	<tr>
	<td>$(B enum)</td>
	<td>$(B enum)</td>
	</tr>

	<tr>
	<td>$(B class)</td>
	<td>no equivalent</td>
	</tr>

	<tr>
	<td>$(I type)$(B *)</td>
	<td>$(I type) $(B *)</td>
	</tr>

	<tr>
	<td>$(I type)$(B [)$(I dim)$(B ])</td>
	<td>$(I type)$(B [)$(I dim)$(B ])</td>
	</tr>

	<tr>
	<td>$(I type)$(B [)$(I dim)$(B ]*)</td>
	<td>$(I type)$(B (*)[)$(I dim)$(B ])</td>
	</tr>

	<tr>
	<td>$(I type)$(B [])</td>
	<td>no equivalent</td>
	</tr>

	<tr>
	<td>$(I type)$(B [)$(I type)$(B ])</td>
	<td>no equivalent</td>
	</tr>

	<tr>
	<td>$(I type) $(B function)$(B $(LPAREN))$(I parameters)$(B $(RPAREN))</td>
	<td>$(I type)$(B (*))$(B $(LPAREN))$(I parameters)$(B $(RPAREN))</td>
	</tr>

	<tr>
	<td>$(I type) $(B delegate)$(B $(LPAREN))$(I parameters)$(B $(RPAREN))</td>
	<td>no equivalent</td>
	</tr>

<!--
	<tr>
	<td>$(B )</td>
	<td>$(B )</td>
	</tr>
-->
</table>
	<p>

	These equivalents hold for most 32 bit C compilers. The C standard
	does not pin down the sizes of the types, so some care is needed.

<h2>Calling printf()</h2>

	This mostly means checking that the printf format specifier
	matches the corresponding D data type.
	Although printf is designed to handle 0 terminated strings,
	not D dynamic arrays of chars, it turns out that since D
	dynamic arrays are a length followed by a pointer to the data,
	the $(TT %.*s) format works perfectly:

------
void foo(char[] string)
{
    printf("my string is: %.*s\n", string);
}
------

	The printf format string literal
	in the example doesn't end with \0. This is because string literals,
	when they are not part of an initializer to a larger data structure,
	have a \0 character helpfully stored after the end of them.
	<p>

	An improved D function for formatted output is
	$(TT std.stdio.writef()).

<h2>Structs and Unions</h2>

	D structs and unions are analogous to C's.
	<p>

	C code often adjusts the alignment and packing of struct members
	with a command line switch or with various implementation specific
	#pragma's. D supports explicit alignment attributes that correspond
	to the C compiler's rules. Check what alignment the C code is using,
	and explicitly set it for the D struct declaration.
	<p>

	D does not support bit fields. If needed, they can be emulated
	with shift and mask operations.

<hr><h1 align=center><u>Interfacing to C++</u></h1>

	D does not provide an interface to C++. Since D, however,
	interfaces directly to C, it can interface directly to
	C++ code if it is declared as having C linkage.
	<p>

	D class objects are incompatible with C++ class objects.

)

Macros:
	TITLE=D Programming Language - Interfacing to C
	WIKI=InterfaceToC

