Ddoc

$(COMMUNITY The C Preprocessor Versus D,

	Back when C was invented, compiler technology was primitive.
	Installing a text
	macro preprocessor onto the front end was a straightforward
	and easy way to add many
	powerful features. The increasing size & complexity of programs
	have illustrated
	that these features come with many inherent problems.
	D doesn't have a preprocessor; but
	D provides a more scalable means to solve the same problems.

$(UL 
	<li><a href="#headerfiles">Header Files</a>
	<li><a href="#pragmaonce">#pragma once</a>
	<li><a href="#pragmapack">#pragma pack</a>
	<li><a href="#macros">Macros</a>
	<li><a href="#conditionalcompilation">Conditional Compilation</a>
	<li><a href="#codefactoring">Code Factoring</a>
	<li><a href="#staticassert">#error and Static Asserts</a>
	<li><a href="#mixins">Mixins</a>
)

<hr><!-- -------------------------------------------- -->
<h3><a name="headerfiles">Header Files</a></h3>

<h4>The C Preprocessor Way</h4>

	C and C++ rely heavily on textual inclusion of header files.
	This frequently results in the compiler having to recompile tens of thousands
	of lines of code over and over again for every source file, an obvious
	source of slow compile times. What header files are normally used for is
	more appropriately done doing a symbolic, rather than textual, insertion.
	This is done with the import statement. Symbolic inclusion means the compiler
	just loads an already compiled symbol table. The needs for macro "wrappers" to
	prevent multiple #inclusion, funky #pragma once syntax, and incomprehensible
	fragile syntax for precompiled headers are simply unnecessary and irrelevant to 
	D.

$(CCODE
#include &lt;stdio.h&gt;
)

<h4>The D Way</h4>

	D uses symbolic imports:

---------
import std.c.stdio;
---------

<hr><!-- -------------------------------------------- -->
<h3><a name="pragmaonce">#pragma once</a></h3>

<h4>The C Preprocessor Way</h4>

	C header files frequently need to be protected against
	being #include'd multiple times.
	To do it, a header file will contain the line:

$(CCODE
#pragma once
)

	or the more portable:

$(CCODE
#ifndef __STDIO_INCLUDE
#define __STDIO_INCLUDE
... header file contents
#endif
)

<h4>The D Way</h4>

	Completely unnecessary since D does a symbolic include of import
	files; they only get imported once no matter how many times
	the import declaration appears.

<hr><!-- -------------------------------------------- -->
<h3><a name="pragmapack">#pragma pack</a></h3>

<h4>The C Preprocessor Way</h4>

	This is used in C to adjust the alignment for structs.

<h4>The D Way</h4>

	For D classes, there is no
	need to adjust the alignment  (in fact, the compiler is free to rearrange the data 
	fields to get the optimum layout, much as the compiler will rearrange local 
	variables on the stack frame). For D structs
	that get mapped onto externally defined data structures, there is a need, and
	it is handled with:

---------
struct Foo
{
	align (4):	// use 4 byte alignment
	...
}
---------

<hr><!-- -------------------------------------------- -->
<h3><a name="macros">Macros</a></h3>

	Preprocessor macros add powerful features and flexibility to C. But they have a
	downside:

$(UL 
	<li> Macros have no concept of scope; they are valid from the point of definition
	to the end of the source. They cut a swath across .h files, nested code, etc. When
	#include'ing tens of thousands of lines of macro definitions, it becomes 
	problematical to avoid inadvertent macro expansions.

	<li> Macros are unknown to the debugger. Trying to debug a program with 
	symbolic data is undermined by the debugger only knowing about macro 
	expansions, not the macros themselves.

	<li> Macros make it impossible to tokenize source code, as an earlier macro change 
	can arbitrarily redo tokens.

	<li> The purely textual basis of macros leads to arbitrary and inconsistent usage,
	making code using macros error prone. (Some attempt to resolve this was 
	introduced with templates in C++.)

	<li> Macros are still used to make up for deficits in the language's expressive
	capability, such as for "wrappers" around header files.
)


	Here's an enumeration of the common uses for macros, and the corresponding 
	feature in D:
	<p>

$(OL 
	<li> Defining literal constants:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#define VALUE	5
)

	<h4>The D Way</h4>

---------
const int VALUE = 5;
---------

	<li> Creating a list of values or flags:

	<h4>The C Preprocessor Way</h4>

$(CCODE
int flags:
#define FLAG_X	0x1
#define FLAG_Y	0x2
#define FLAG_Z	0x4
...
flags |= FLAG_X;
)

	<h4>The D Way</h4>

---------
enum FLAGS { X = 0x1, Y = 0x2, Z = 0x4 };
FLAGS flags;
...
flags |= FLAGS.X;
---------

	<li> Distinguishing between ascii chars and wchar chars:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#if UNICODE
    #define dchar	wchar_t
    #define TEXT(s)	L##s
#else
    #define dchar	char
    #define TEXT(s)	s
#endif

...
dchar h[] = TEXT("hello");
)

	<h4>The D Way</h4>

---------
dchar[] h = "hello";
---------


	D's optimizer will inline the function, and will do the conversion of the
	string constant at compile time.
	<p>

	<li> Supporting legacy compilers:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#if PROTOTYPES
#define P(p)	p
#else
#define P(p)	()
#endif
int func P((int x, int y));
)

	<h4>The D Way</h4>

	By making the D compiler open source, it will largely
	avoid the problem of syntactical backwards compatibility.

	<li> Type aliasing:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#define INT 	int
)

	<h4>The D Way</h4>

---------
alias int INT;
---------

	<li> Using one header file for both declaration and definition:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#define EXTERN extern
#include "declarations.h"
#undef EXTERN
#define EXTERN
#include "declarations.h"
)

	In declarations.h:

$(CCODE
EXTERN int foo;
)


	<h4>The D Way</h4>

	The declaration and the definition are the same, so there is no need
	to muck with the storage class to generate both a declaration and a definition
	from the same source.

	<li> Lightweight inline functions:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#define X(i)	((i) = (i) / 3)
)

	<h4>The D Way</h4>

---------
int X(inout int i) { return i = i / 3; }
---------

	The compiler optimizer will inline it; no efficiency is lost.

	<li> Assert function file and line number information:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#define assert(e)	((e) || _assert(__LINE__, __FILE__))
)

	<h4>The D Way</h4>

	assert() is a built-in expression primitive. Giving the compiler
	such knowledge of assert() also enables the optimizer to know about things
	like the _assert() function never returns.

	<li> Setting function calling conventions:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#ifndef _CRTAPI1
#define _CRTAPI1 __cdecl
#endif
#ifndef _CRTAPI2
#define _CRTAPI2 __cdecl
#endif

int _CRTAPI2 func();
)

	<h4>The D Way</h4>

	Calling conventions can be specified in blocks, so there's no
	need to change it for every function:

---------
extern (Windows)
{
    int onefunc();
    int anotherfunc();
}
---------

	<li> Hiding __near or __far pointer weirdness:

	<h4>The C Preprocessor Way</h4>

$(CCODE
#define LPSTR	char FAR *
)

	<h4>The D Way</h4>

	D doesn't support 16 bit code, mixed pointer sizes, and different
	kinds of pointers, and so the problem is just
	irrelevant.

	<li> Simple generic programming:

	<h4>The C Preprocessor Way</h4>

	Selecting which function to use based on text substitution:

$(CCODE
#ifdef UNICODE
int getValueW(wchar_t *p);
#define getValue getValueW
#else
int getValueA(char *p);
#define getValue getValueA
#endif
)

	<h4>The D Way</h4>

	D enables declarations of symbols that are $(I aliases) of
	other symbols:

---------
version (UNICODE)
{
    int getValueW(wchar[] p);
    alias getValueW getValue;
}
else
{
    int getValueA(char[] p);
    alias getValueA getValue;
}
---------

)

<hr><!-- -------------------------------------------- -->
<h3><a name="conditionalcompilation">Conditional Compilation</a></h3>


<h4>The C Preprocessor Way</h4>

	Conditional compilation is a powerful feature of the C preprocessor,
	but it has its downside:

    $(UL 
	<li> The preprocessor has no concept of scope. #if/#endif can be
	interleaved with code in a completely unstructured and disorganized
	fashion, making things difficult to follow.

	<li> Conditional compilation triggers off of macros - macros that
	can conflict with identifiers used in the program.

	<li> #if expressions are evaluated in subtly different ways than
	C expressions are.

	<li> The preprocessor language is fundamentally different in concept
	than C, for example, whitespace and line terminators mean things to
	the preprocessor that they do not in C.

    )

<h4>The D Way</h4>

	D supports conditional compilation:

    $(OL 
	<li> Separating version specific functionality into separate modules.

	<li> The debug statement for enabling/disabling debug harnesses,
	extra printing, etc.

	<li> The version statement for dealing with multiple versions
	of the program generated from a single set of sources.

	<li> The if (0) statement.

	<li> The /+ +/ nesting comment can be used to comment out blocks
	of code.

    )

<hr><!-- -------------------------------------------- -->
<h3><a name="codefactoring">Code Factoring</a></h3>

<h4>The C Preprocessor Way</h4>

	It's common in a function to have a repetitive sequence
	of code to be executed in multiple places. Performance
	considerations preclude factoring it out into a separate
	function, so it is implemented as a macro. For example,
	consider this fragment from a byte code interpreter:

$(CCODE
unsigned char *ip;	// byte code instruction pointer
int *stack;
int spi;		// stack pointer
...
#define pop()		(stack[--spi])
#define push(i)		(stack[spi++] = (i))
while (1)
{
    switch (*ip++)
    {
	case ADD:
	    op1 = pop();
	    op2 = pop();
	    result = op1 + op2;
	    push(result);
	    break;

	case SUB:
	...
    }
}
)

	This suffers from numerous problems:
	$(OL 
	<li> The macros must evaluate to expressions and cannot declare
	any variables. Consider the difficulty of extending them to
	check for stack overflow/underflow.
	<li> The macros exist outside of the semantic symbol table, so
	remain in scope even outside of the function they are declared in.
	<li> Parameters to macros are passed textually, not by value,
	meaning that the macro implementation needs to be careful to not
	use the parameter more than once, and must protect it with ().
	<li> Macros are invisible to the debugger, which sees only the
	expanded expressions.
	)

<h4>The D Way</h4>

	D neatly addresses this with nested functions:

---------
ubyte* ip;		// byte code instruction pointer
int[] stack;		// operand stack
int spi;		// stack pointer
...

int pop()        { return stack[--spi]; }
void push(int i) { stack[spi++] = i; }

while (1)
{
    switch (*ip++)
    {
	case ADD:
	    op1 = pop();
	    op2 = pop();
	    push(op1 + op2);
	    break;

	case SUB:
	...
    }
}
---------

	The problems addressed are:
	$(OL 
	<li> The nested functions have available the full expressive
	power of D functions. The array accesses already are bounds
	checked (adjustable by compile time switch).
	<li> Nested function names are scoped just like any other name.
	<li> Parameters are passed by value, so need to worry about
	side effects in the parameter expressions.
	<li> Nested functions are visible to the debugger.
	)

	Additionally, nested functions can be inlined by the implementation
	resulting in the same high performance that the C macro version
	exhibits.

<hr><!-- -------------------------------------------- -->
<h3><a name="staticassert">#error and Static Asserts</a></h3>

	Static asserts are user defined checks made at compile time;
	if the check fails the compile issues an error and fails.

<h4>The C Preprocessor Way</h4>

	The first way is to use the $(TT #error) preprocessing directive:

$(CCODE
#if FOO || BAR
    ... code to compile ...
#else
#error "there must be either FOO or BAR"
#endif
)

	This has the limitations inherent in preprocessor expressions
	(i.e. integer constant expressions only, no casts, no $(TT sizeof),
	no symbolic constants, etc.).
	<p>

	These problems can be circumvented to some extent by defining a
	$(TT static_assert) macro (thanks to M. Wilson):

$(CCODE
#define static_assert(_x) do { typedef int ai[(_x) ? 1 : 0]; } while(0)
)

	and using it like:

$(CCODE
void foo(T t)
{
    static_assert(sizeof(T) < 4);
    ...
}
)

	This works by causing a compile time semantic error if the condition
	evaluates
	to false. The limitations of this technique are a sometimes very
	confusing error message from the compiler, along with an inability
	to use a $(TT static_assert) outside of a function body.

<h4>The D Way</h4>

	D has the <a href="version.html#staticassert">static assert</a>,
	which can be used anywhere a declaration
	or a statement can be used. For example:

---------
version (FOO)
{
    class Bar
    {
	const int x = 5;
	static assert(Bar.x == 5 || Bar.x == 6);

	void foo(T t)
	{
	    static assert(T.sizeof < 4);
	    ...
	}
    }
}
else version (BAR)
{
    ...
}
else
{
    static assert(0);	// unsupported version
}
---------

<hr><!-- -------------------------------------------- -->
<h3><a name="mixins">Mixins</a></h3>

	D <a href="mixin.html">mixins</a> superficially look just
	like using C's preprocessor to insert blocks of code and
	parse them in the scope of where they are instantiated.
	But the advantages of mixins over macros are:

	$(OL 
	<li> Mixins substitute in parsed declaration trees that pass muster with
	the language syntax, macros substitute in arbitrary preprocessor tokens
	that have no organization.

	<li> Mixins are in the same language. Macros are a separate and
	distinct language layered on top of C++, with its own expression rules,
	its own types, its distinct symbol table, its own scoping rules, etc. 

	<li> Mixins are selected based on partial specialization rules, macros
	have no overloading.

	<li> Mixins create a scope, macros do not.

	<li> Mixins are compatible with syntax parsing tools, macros are not.

	<li> Mixin semantic information and symbol tables are passed through to
	the debugger, macros are lost in translation.

	<li> Mixins have override conflict resolution rules, macros just
	collide.

	<li> Mixins automatically create unique identifiers as required using a
	standard algorithm, macros have to do it manually with kludgy token
	pasting.

	<li> Mixin value arguments with side effects are evaluated once, macro
	value arguments get evaluated each time they are used in the expansion
	(leading to weird bugs).

	<li> Mixin argument replacements don't need to be 'protected' with
	parentheses to avoid operator precedence regrouping.

	<li> Mixins can be typed as normal D code of arbitrary length, multiline
	macros have to be backslash line-spliced, can't use // to end of line
	comments, etc.

	<li> Mixins can define other mixins. Macros cannot create other macros.

	)

)

Macros:
	TITLE=Digital Mars - The C Preprocessor vs D
	WIKI=PreToD

